#include <windows.h>
#include <GL/gl.h>
#include <GL/wglext.h>
#include "glcontext.h"
#include "window_private.h"

using namespace std;

namespace Msp {
namespace Graphics {

typedef HGLRC ContextHandle;

struct GLContext::Private
{
	ContextHandle context;
};


void GLContext::platform_init(const GLOptions &opts)
{
	HDC dc = GetDC(window.get_private().window);

	PIXELFORMATDESCRIPTOR pfd;
	memset(&pfd, 0, sizeof(pfd));

	pfd.nSize = sizeof(pfd);
	pfd.nVersion = 1;
	pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
	if(opts.doublebuffer)
		pfd.dwFlags |= PFD_DOUBLEBUFFER;
	pfd.iPixelType = PFD_TYPE_RGBA;
	if(opts.alpha)
		pfd.cAlphaBits = 1;
	pfd.cDepthBits = 1;
	if(opts.stencil)
		pfd.cStencilBits = 1;

	int pf_index = ChoosePixelFormat(dc, &pfd);
	if(!pf_index)
		throw unsupported_gl_mode(opts);
	SetPixelFormat(dc, pf_index, &pfd);

	priv = new Private;
	if(opts.forward_compatible || opts.gl_version_major)
	{
		PFNWGLCREATECONTEXTATTRIBSARBPROC wglCreateContextAttribs = reinterpret_cast<PFNWGLCREATECONTEXTATTRIBSARBPROC>(wglGetProcAddress("wglCreateContextAttribs"));
		if(!wglCreateContextAttribs)
			throw unsupported_gl_mode(opts);

		vector<int> ctx_attribs;

		if(opts.forward_compatible)
		{
			ctx_attribs.push_back(WGL_CONTEXT_FLAGS_ARB);
			ctx_attribs.push_back(WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB);
		}

		if(opts.gl_version_major)
		{
			ctx_attribs.push_back(WGL_CONTEXT_MAJOR_VERSION_ARB);
			ctx_attribs.push_back(opts.gl_version_major);
			ctx_attribs.push_back(WGL_CONTEXT_MINOR_VERSION_ARB);
			ctx_attribs.push_back(opts.gl_version_minor);
		}

		ctx_attribs.push_back(0);

		priv->context = wglCreateContextAttribs(dc, 0, &ctx_attribs[0]);
	}
	else
		priv->context = wglCreateContext(dc);
	wglMakeCurrent(dc, priv->context);

	ReleaseDC(window.get_private().window, dc);
}

GLContext::~GLContext()
{
	wglMakeCurrent(0, 0);
	wglDeleteContext(priv->context);

	delete priv;
}

void GLContext::set_swap_interval(unsigned i)
{
	PFNWGLSWAPINTERVALEXTPROC wglSwapInterval = reinterpret_cast<PFNWGLSWAPINTERVALEXTPROC>(wglGetProcAddress("wglSwapIntervalEXT"));
	if(!wglSwapInterval)
		throw runtime_error("wglSwapIntervalEXT not found");
	wglSwapInterval(i);
}

void GLContext::swap_buffers()
{
	HDC dc = GetDC(window.get_private().window);
	SwapBuffers(dc);
	ReleaseDC(window.get_private().window, dc);
}

void GLContext::window_resized(unsigned, unsigned)
{ }

} // namespace Graphics
} // namespace Msp
