#include <stdexcept>
#include <sigc++/bind.h>
#include <sigc++/bind_return.h>
#include <msp/core/hash.h>
#include <msp/core/maputils.h>
#include "hub.h"

using namespace std;

namespace Msp {
namespace Input {

Hub::Hub()
{
	name = "Hub";
}

void Hub::attach(Device &dev)
{
	unsigned tag = hash32(dev.get_name(), 20);
	while(devices.count(tag))
		++tag;

	devices[tag] = &dev;
	dev.signal_button_press.connect(sigc::bind_return(sigc::bind(sigc::mem_fun(this, &Hub::button_press), tag), false));
	dev.signal_button_release.connect(sigc::bind_return(sigc::bind(sigc::mem_fun(this, &Hub::button_release), tag), false));
	dev.signal_axis_motion.connect(sigc::bind_return(sigc::bind(sigc::mem_fun(this, &Hub::axis_motion), tag), false));
}

std::string Hub::get_button_name(unsigned btn) const
{
	const Device &dev = *get_item(devices, btn>>12);
	return dev.get_name()+": "+dev.get_button_name(btn&0xFFF);
}

std::string Hub::get_axis_name(unsigned axis) const
{
	const Device &dev = *get_item(devices, axis>>12);
	return dev.get_name()+": "+dev.get_axis_name(axis&0xFFF);
}

void Hub::button_press(unsigned btn, unsigned tag)
{
	set_button_state((tag<<12) | (btn&0xFFF), true, true);
}

void Hub::button_release(unsigned btn, unsigned tag)
{
	set_button_state((tag<<12) | (btn&0xFFF), false, true);
}

void Hub::axis_motion(unsigned axis, float value, float, unsigned tag)
{
	set_axis_value((tag<<12) | (axis&0xFFF), value, true);
}

} // namespace Input
} // namespace Msp
